/**
 *	S3C (Simple Server-side Cache)
 *	Server-side Object Caching Framework 
 *
 *	Copyright (C) 2005  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	Contact Info:
 *	Email: xeus.man@gmail.com
 */

package xeus.s3c;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import xeus.s3c.event.SlotEvent;
import xeus.s3c.event.SlotListener;

/**
 * @author Kamran Zafar
 *  
 */
public class Slot implements Serializable {
    private HashMap slot;

    private String name;

    private int timeout;

    private List listeners;

    private Invalidator iThread;
    
    public Slot(String name, int timeout) {
        this(name);
        this.timeout = timeout;
    }

    public Slot(String name) {
        slot = new HashMap();
        listeners = new ArrayList();
        this.name = name;
        this.timeout = -1;
    }

    public void initialize() {
        if (timeout > 0)
            (new Invalidator(this)).start();
    }
    
    public void invalidate()
    {
        if(iThread!=null)
            iThread.interrupt();
        
        fireSlotInvalidatedEvent(new SlotEvent(this));
    }
    
    public void updated()
    {
        fireSlotUpdatedEvent(new SlotEvent(this));
    }

    public void put(String key, Object value) {
        slot.put(key, value);
        
        SlotEvent event=new SlotEvent(this,key);
        fireElementAddedEvent(event);
    }

    public Object get(String key) {
        return slot.get(key);
    }

    public Collection values() {
        return slot.values();
    }

    public void remove(String key) {
        slot.remove(key);
        
        SlotEvent event=new SlotEvent(this,key);
        fireElementRemovedEvent(event);
    }
    
    public void empty()
    {
        slot.clear();
        fireSlotEmptiedEvent(new SlotEvent(this));
    }

    public void addSlotListener(SlotListener listener) {
        listeners.add(listener);
    }

    public void removeSlotListener(SlotListener listener) {
        listeners.remove(listener);
    }

    protected void fireSlotEmptiedEvent(SlotEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((SlotListener) listeners.get(i)).slotEmptied(event);
    }
    
    protected void fireElementRemovedEvent(SlotEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((SlotListener) listeners.get(i)).elementRemoved(event);
    }
    
    protected void fireElementAddedEvent(SlotEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((SlotListener) listeners.get(i)).elementAdded(event);
    }
    
    protected void fireSlotInvalidatedEvent(SlotEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((SlotListener) listeners.get(i)).slotInvalidated(event);
    }

    protected void fireSlotUpdatedEvent(SlotEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((SlotListener) listeners.get(i)).slotUpdated(event);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    private class Invalidator extends Thread implements Serializable {
        Slot slot;

        public Invalidator(Slot slot) {
            this.slot = slot;
        }

        public void run() {
            while (true) {
                try {
                    sleep(slot.getTimeout());
                } catch (InterruptedException e) {
                    continue;
                }
                slot.fireSlotInvalidatedEvent(new SlotEvent(slot));
            }
        }
    }
}